package com.lutao.carlocation.hand;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.lutao.carlocation.entity.CollectLog;
import com.lutao.carlocation.kafka.RsaBase64Utils;
import com.lutao.carlocation.service.CollectLogService;
import com.lutao.carlocation.service.IsolatedVehicleLocationMonitorService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.config.SaslConfigs;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Properties;

@Service
@Slf4j
public class KafkaConsumerService {

    @Resource(name = "kafkaTemplate2")
    private KafkaTemplate<String, String> kafkaTemplate;

    @Resource
    private IsolatedVehicleLocationMonitorService isolatedVehicleLocationMonitorService;

    @Resource
    private CollectLogService collectLogService;




    public void startConsuming() {
        // 初始化消费者属性
        Properties props = new Properties();
        // 设置服务器地址、组ID等...
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, "datapoc1.66yunlian.cn:30886,datapoc2.66yunlian.cn:30886,datapoc3.66yunlian.cn:30886");
        props.put(ConsumerConfig.GROUP_ID_CONFIG, "topic_push_out_jinta_group");
        props.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, true);
        props.put(ConsumerConfig.AUTO_COMMIT_INTERVAL_MS_CONFIG, "1000");
        props.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "earliest");
        props.put(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, "30000");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        props.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, "SASL_PLAINTEXT");
        props.put(SaslConfigs.SASL_MECHANISM, "SCRAM-SHA-256");
        props.put(SaslConfigs.SASL_JAAS_CONFIG, "org.apache.kafka.common.security.scram.ScramLoginModule required username=\"customer67\" password=\"dscPEQAnyJa%^ZwK\";");
        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
        consumer.subscribe(Arrays.asList("topic_push_out_jinta"));
        while (true) {
            ConsumerRecords<String, String> records = consumer.poll(100);
            for (ConsumerRecord<String, String> t : records) {
                log.info(t.key() + "===" + t.value());
                try {
                    switch (t.key()){
                        case "gps_common_vehicle_loc":
                            //可视化常用车位置数据
                            break;
                        case "gps_vehicle_location":
                            //车辆位置信息数据
                            String decrypt = RsaBase64Utils.decrypt(t.value());
                            if (JSONUtil.isJson(decrypt)) {
                                JSONObject receiveMessage = JSONUtil.parseObj(decrypt);
                                log.info("received gps_vehicle_location message:{}", receiveMessage);
                                JSONObject sendMessage = JSONUtil.createObj();
                                sendMessage.putOnce("locaterCode", receiveMessage.get("vehicleNO"));
                                sendMessage.putOnce("longitude", receiveMessage.get("longitude"));
                                sendMessage.putOnce("latitude", receiveMessage.get("latitude"));
                                Integer positionTime = (Integer) receiveMessage.get("positionTime");
                                long time = positionTime * 1000L;
                                // 使用Instant.ofEpochMilli()方法将时间戳转换为Instant对象
                                Instant instant = Instant.ofEpochMilli(time);
                                // 指定时区，例如使用UTC时区
                                ZonedDateTime zonedDateTime = instant.atZone(ZoneId.systemDefault());
                                // 创建DateTimeFormatter对象，并指定输出的日期格式
                                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                                // 使用DateTimeFormatter对象将ZonedDateTime对象转换为字符串
                                String formattedDate = zonedDateTime.format(formatter);
                                sendMessage.putOnce("positioningTime", formattedDate);
                                sendMessage.putOnce("address", "-");
                                sendMessage.putOnce("speed", receiveMessage.get("vec"));
                                sendMessage.putOnce("vehicleNO", receiveMessage.get("vehicleNO"));
                                sendMessage.putOnce("plateColor", receiveMessage.get("plateColor"));
                                sendMessage.putOnce("altitude", receiveMessage.get("alititude"));
                                sendMessage.putOnce("mileage", receiveMessage.get("mileage"));
                                sendMessage.putOnce("direction", receiveMessage.get("direction"));
                                sendMessage.putOnce("accState", receiveMessage.get("accState"));
                                sendMessage.putOnce("positionTime", receiveMessage.get("positionTime"));
                                sendMessage.putOnce("directionDesc", receiveMessage.get("directionDesc"));
                                sendMessage.putOnce("vec", receiveMessage.get("vec"));
                                sendMessage.putOnce("online_status", receiveMessage.get("onlineStatus"));
                                sendMessage.putOnce("transport_status", receiveMessage.get("transportStatus"));
                                JSONArray jsonArray = JSONUtil.createArray();
                                jsonArray.add(JSONUtil.parse(sendMessage));
                                kafkaTemplate.send("vehicleLocation", jsonArray.toString());
                                collectLogService.save(new CollectLog().setType("gps_vehicle_location").setContent(jsonArray.toString()));
                                log.info("send gps_vehicle_location message:{}", jsonArray);
                                //记录
                                isolatedVehicleLocationMonitorService.saveSendMessage(sendMessage);
                            }
                            break;
                        case "park_filing_info":
                            break;
                        case "park_filing_queue_log":
                            break;
                        case "fence_log":
                            break;
                        case "alarm_log":
                            //报警日志数据
                            try {
                                collectLogService.save(new CollectLog().setType("alarmLog").setContent(t.value()));
                                String alarm = RsaBase64Utils.decrypt(t.value());
                                collectLogService.save(new CollectLog().setType("alarmLog").setContent(alarm));
                                if (JSONUtil.isJson(alarm)) {
                                    JSONObject receiveMessage = JSONUtil.parseObj(alarm);
                                    log.info("received alarm_log message:{}", receiveMessage);
                                    kafkaTemplate.send("alarmLog", receiveMessage.toString());
                                    log.info("send alarm_log message:{}", receiveMessage);
                                    //collectLogService.save(new CollectLog().setType("alarmLog").setContent(receiveMessage.toString()));
                                }
                            } catch (Exception e) {
                                log.error("解析异常车辆报警异常，exception:{}", e);
                                collectLogService.save(new CollectLog().setType("error-车辆报警").setContent(t.value()));
                            }
                            break;
                        case "park_view_count":
                            //报警车辆数
                            break;
                        case "park_filing_queue_count":
                            //报备队列统计数据
                            break;
                    }
                }catch (Exception e){
                    log.error("消费数据异常",e);
                }
            }
        }
    }
}
